@@ -1,8 +1,10 @@
 import torch
 from torch import nn
-from inspect import isfunction
 import torch.nn.functional as F
 
+import math
+from inspect import isfunction
+
 # constants
 
 MIN_EXPERT_CAPACITY = 4
@@ -13,6 +15,11 @@ def default(val, default_val):
     default_val = default_val() if isfunction(default_val) else default_val
     return val if val is not None else default_val
 
+def cast_tuple(el):
+    return el if isinstance(el, tuple) else (el,)
+
+# tensor related helper functions
+
 def top1(t):
     values, index = t.topk(k=1, dim=-1)
     values, index = map(lambda x: x.squeeze(dim=-1), (values, index))
@@ -26,13 +33,20 @@ def cumsum_exclusive(t, dim=-1):
     padded_t = F.pad(t, (*pre_padding, 1, 0)).cumsum(dim=dim)
     return padded_t[(..., slice(None, -1), *pre_slice)]
 
-def cast_tuple(el):
-    return el if isinstance(el, tuple) else (el,)
-
+# pytorch one hot throws an error if there are out of bound indices.
+# tensorflow, in contrast, does not throw an error
 def safe_one_hot(indexes, max_length):
     max_index = indexes.max() + 1
     return F.one_hot(indexes, max(max_index + 1, max_length))[..., :max_length]
 
+# activations
+
+class GELU_(nn.Module):
+    def forward(self, x):
+        return 0.5 * x * (1 + torch.tanh(math.sqrt(2 / math.pi) * (x + 0.044715 * torch.pow(x, 3))))
+
+GELU = nn.GELU if hasattr(nn, 'GELU') else GELU_
+
 # expert class
 
 class Experts(nn.Module):
@@ -40,7 +54,7 @@ class Experts(nn.Module):
         dim,
         num_experts = 16,
         hidden_dim = None,
-        activation = nn.ReLU):
+        activation = GELU):
         super().__init__()
 
         hidden_dim = default(hidden_dim, dim * 4)
